@@ -1,17 +1,18 @@
-#!/bin/python
 # -*- coding: utf-8 -*-
+"""functions for the one-asset HANK model without labor choice. Heavily inspired by https://github.com/shade-econ/sequence-jacobian/#sequence-space-jacobian
+"""
 
 import jax
 import jax.numpy as jnp
-from grgrlib.jaxed import jax_print
+from grgrjax import jax_print
 from econpizza.utilities.interp import interpolate
 
 
-def hh_init(a_grid, e_grid):
-    return jnp.ones((e_grid.shape[0], a_grid.shape[0]))*1e-2
+def hh_init(a_grid, skills_grid):
+    return jnp.ones((skills_grid.shape[0], a_grid.shape[0]))*1e-2
 
 
-def hh(Va_p, a_grid, skills_grid, w, n, T, R, beta, eis, frisch):
+def hh(Va_p, a_grid, skills_grid, w, n, T, R, beta, sigma_c, sigma_l):
     """A single backward step via EGM
     """
 
@@ -20,7 +21,7 @@ def hh(Va_p, a_grid, skills_grid, w, n, T, R, beta, eis, frisch):
 
     # consumption can be readily obtained from MUC and MU of labor
     labor_inc = skills_grid[:, None]*n*w
-    c_nextgrid = ux_nextgrid**-eis + labor_inc/(1 + 1/frisch)
+    c_nextgrid = ux_nextgrid**(-1/sigma_c) + labor_inc/(1 + sigma_l)
 
     # get consumption in grid space
     lhs = c_nextgrid - labor_inc + a_grid[None, :] - T[:, None]
@@ -37,15 +38,15 @@ def hh(Va_p, a_grid, skills_grid, w, n, T, R, beta, eis, frisch):
     a = jnp.where(a < a_grid[0], a_grid[0], a)
 
     # calculate new MUC
-    Va = R * (c - labor_inc/(1 + 1/frisch)) ** (-1 / eis)
+    Va = R * (c - labor_inc/(1 + sigma_l)) ** (-sigma_c)
 
     return Va, a, c
 
 
-def transfers(age_stationary, Div, Tax, e_grid):
+def transfers(skills_stationary, Div, Tax, skills_grid):
     # hardwired incidence rules are proportional to skill; scale does not matter
-    rule = e_grid
-    div = Div / jnp.sum(age_stationary * rule) * rule
-    tax = Tax / jnp.sum(age_stationary * rule) * rule
+    rule = skills_grid
+    div = Div / jnp.sum(skills_stationary * rule) * rule
+    tax = Tax / jnp.sum(skills_stationary * rule) * rule
     T = div - tax
     return T
