@@ -1,40 +1,10 @@
-#!/bin/python
-# -*- coding: utf-8 -*-
+"""Tools for dealing with distributions
+"""
 
 import jax
 import jax.numpy as jnp
 
 
-def tmat_from_exog(probs, D):
-
-    nZ, nX = D.shape
-    tmat = jnp.zeros((nX*nZ, nX*nZ))
-
-    i = jnp.arange(nZ)*nX
-    xi = jnp.arange(nX)[:, None]
-    zi = jnp.arange(nZ)[:, None, None]
-    tmat = tmat.at[xi+zi*nX, xi +
-                   i].set(probs.broadcast_in_dim((nZ, nX, nZ), (0, 2)))
-
-    return tmat
-
-
-def tmat_from_endo(x_i, probs):
-
-    nZ, nX = x_i.shape
-    tmat = jnp.zeros((nX*nZ, nX*nZ))
-
-    ix = jnp.arange(nX*nZ)
-    j = jnp.arange(nZ).repeat(nX)*nX
-    i = x_i.ravel()
-    pi = probs.ravel()
-
-    tmat = tmat.at[i+j, ix].add(pi)
-    tmat = tmat.at[i+1+j, ix].add(1-pi)
-
-    return tmat
-
-
 def expect_transition(Pi, X):
     """If Pi is a matrix, multiply Pi times the ith dimension of X and return"""
 
@@ -60,22 +30,27 @@ def forward_policy_1d(D, x_i, x_pi):
     return Dnew
 
 
+def cond_func(carry):
+    (dist, cnt, dist_old), (tol, maxit), _ = carry
+    cond0 = jnp.abs(dist-dist_old).max() > tol
+    cond1 = cnt < maxit
+    return jnp.logical_and(cond0, cond1)
+
+
+def _body_func_1d(carry):
+    (dist, cnt, _), cond_vars, exo_endo = carry
+    exog_probs, endog_inds, endog_probs = exo_endo
+    dist_new = exog_probs.T @ forward_policy_1d(dist, endog_inds, endog_probs)
+    return (dist_new, cnt + 1, dist), cond_vars, exo_endo
+
+
 def stationary_distribution_forward_policy_1d(endog_inds, endog_probs, exog_probs, tol=1e-10, maxit=1000):
 
     dist = jnp.ones_like(endog_inds, dtype=jnp.float64)
     dist /= dist.sum()
 
-    def cond_func(cont):
-        dist, dist_old, cnt = cont
-        cond0 = jnp.abs(dist-dist_old).max() > tol
-        cond1 = cnt < maxit
-        return cond0 & cond1
-
-    def body_func(cont):
-        dist, _, cnt = cont
-        return exog_probs.T @ forward_policy_1d(dist, endog_inds, endog_probs), dist, cnt + 1
-
-    dist, _, cnt = jax.lax.while_loop(cond_func, body_func, (dist, dist+1, 0))
+    (dist, cnt, _), _, _ = jax.lax.while_loop(
+        cond_func, _body_func_1d, ((dist, 0, dist+1), (tol, maxit), (exog_probs, endog_inds, endog_probs)))
     return dist, cnt
 
 
@@ -94,32 +69,31 @@ def forward_policy_2d(D, x_i, y_i, x_pi, y_pi):
     return Dnew
 
 
+def _body_func_2d(carry):
+    (dist, cnt, _), cond_vars, exo_endo = carry
+    exog_probs, endog_inds0, endog_inds1, endog_probs0, endog_probs1 = exo_endo
+    pre_exo_dist = forward_policy_2d(
+        dist, endog_inds0, endog_inds1, endog_probs0, endog_probs1)
+    new_dist = expect_transition(exog_probs.T, pre_exo_dist)
+    return (new_dist, cnt + 1, dist), cond_vars, exo_endo
+
+
 def stationary_distribution_forward_policy_2d(endog_inds0, endog_inds1, endog_probs0, endog_probs1, exog_probs, tol=1e-10, maxit=1000):
     # TODO: can be merged with stationary_distribution_forward_policy_1d
 
     dist = jnp.ones_like(endog_inds0, dtype=jnp.float64)
     dist /= dist.sum()
 
-    def cond_func(cont):
-        dist, dist_old, cnt = cont
-        cond0 = jnp.abs(dist-dist_old).max() > tol
-        cond1 = cnt < maxit
-        return cond0 & cond1
+    exo_endo = exog_probs, endog_inds0, endog_inds1, endog_probs0, endog_probs1
+    (dist, cnt, _), _, _ = jax.lax.while_loop(cond_func,
+                                              _body_func_2d, ((dist, 0, dist+1), (tol, maxit), exo_endo))
 
-    def body_func(cont):
-        dist, _, cnt = cont
-        pre_exo_dist = forward_policy_2d(
-            dist, endog_inds0, endog_inds1, endog_probs0, endog_probs1)
-        new_dist = expect_transition(exog_probs.T, pre_exo_dist)
-        return new_dist, dist, cnt + 1
-
-    dist, _, cnt = jax.lax.while_loop(cond_func, body_func, (dist, dist+1, 0))
     return dist, cnt
 
 
 def stationary_distribution(T):
     """Find invariant distribution of a Markov chain by unit eigenvector.
-    NOTE: jax has no autodiff support for eig.
+    NOTE: jax has no autodiff support for eig. (there is a version with custom_jvp in grgrwip)
     """
 
     v, w = jnp.linalg.eig(T)
